#Ke.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np


import json
import datetime

from enum import Enum

class BondType(Enum):
 USTreasury10Year = "USTreasury10Year"
 ThaiGovernmentBond10Year   = "ThaiGovernmentBond10Year"
 USTreasury10YearAnd2Year = "USTreasury10YearAnd2Year"



def validate(date_string):
 date_format = '%Y-%m-%d'
 try:
  date_obj = datetime.datetime.strptime(date_string, date_format)
  return 1
 except ValueError:
  print("Incorrect data format, should be YYYY-MM-DD")
  return 0


#http://www.thaibma.or.th/EN/Market/YieldCurve/Government.aspx
def thaibma(date="",bondtype="",viewlog=True):  
 if(date==""):
    date = datetime.datetime.now().strftime("%Y-%m-%d")

 if(validate(date)==0):
     return 0
 
 if(bondtype==""):
    bondtype = BondType.USTreasury10Year
 
 if(not isinstance(bondtype, BondType)):
    print("dataType Not Match")
    return 0,0
  
 bondtype_ = bondtype.value
  
 if(bondtype_=="USTreasury10Year" or bondtype_=="USTreasury10YearAnd2Year"): 
    jsonKey = "USTreasury"  
    url = "http://www.thaibma.or.th/yieldcurve/uszyc/"+date

 if(bondtype_=="ThaiGovernmentBond10Year"):
    jsonKey = "Curve"  
    url = "http://www.thaibma.or.th/yieldcurve/gov/"+date

 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
 r = requests.get(url,headers=headers)
 r.content
 soup = BeautifulSoup(r.content, "lxml") 
    
 data  = json.loads(soup.text)  
 try: 
  if(bondtype_=="USTreasury10Year"):
   for i in data[jsonKey]:     
    if((str(i['X']))=="10.0"):
     return ((float(i['Y']))/100)
  
  if(bondtype_=="ThaiGovernmentBond10Year"):
   for i in data[jsonKey]:     
    if((str(i['X']))=="10.0"):
     return ((float(i['Y']))/100)

  if(bondtype_=="USTreasury10YearAnd2Year"):  
    p = 0
    k2 = 0 
    k10 = 0
    for i in data[jsonKey]:  
      if((str(i['X']))=="2.0"):  
         p = p+1  
         k2 = ((float(i['Y']))/100)
      if((str(i['X']))=="10.0"):
         p = p+1 
         k10 = ((float(i['Y']))/100)
      if(p==2):
         return {"USTreasury2Year":round(k2,4),"USTreasury10Year":round(k10,4)}
 except:
    if(viewlog==True):
      print("ไม่พบข้อมูล")
    return 0


#http://www.thaibma.or.th/EN/Market/YieldCurve/Government.aspx
def governmentBond10year(date=""):  
 if(date==""):
   date = datetime.datetime.now().strftime("%Y-%m-%d")

 if(validate(date)==0):
     return 0
    
 url = "http://www.thaibma.or.th/yieldcurve/gov/"+date
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
 r = requests.get(url,headers=headers)
 r.content
 soup = BeautifulSoup(r.content, "lxml") 
    
 data  = json.loads(soup.text)   
 for i in data['Curve']:     
  if((str(i['X']))=="10.0"):
    return ((float(i['Y']))/100)
 
 print("ไม่พบข้อมูล")
 return 0


def findIndexData(strfind,g_data):
  indexat = 0
  cnt = 0
  for i in (g_data):
    if(strfind in i.text):
        indexat = cnt
    cnt+=1
    
  return indexat  

def beta(symbol):
 #ป้องกันพวก &
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
 mrkCap = np.NAN
 symbolK = symbol.replace('&', '%26')
 r = requests.get("https://www.set.or.th/set/factsheet.do?symbol="+symbolK+"&ssoPageId=3&language=th&country=TH",headers=headers)
 r.content
 soup = BeautifulSoup(r.content, "lxml")  

 g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})#[2].find_all("td")
 if(len(g_data)==0):
   print("ไม่พบข้อมูล")  
   return 0
 
 ##########################################
 indexat = findIndexData("ข้อมูลสถิติ",g_data)
 tr_ds = g_data[indexat].find_all("tr")
 ##########################################
    
    
 th_ds = []   
 data_ds = []
 for i in tr_ds:
    tddata = i.find_all("td")
    #print(tddata[0].text)
    if("ข้อมูลสถิติ" in tddata[0].text):
       for k in tddata:
          th_ds.append(k.text)
    if("Beta" in tddata[0].text):
       z = 0 
       for k in tddata:
          m = k.text  
          if(z>0):  
            if("-" in m):
                print("ไม่พบข้อมูล")
                return 0
            m = float(m.replace(",",""))
          data_ds.append(m)     
          z+=1
            
 ##########################################
 df = pd.DataFrame({"ind":th_ds,"ds":data_ds}).T
 df.columns = th_ds   
 df = df.drop(df[df["ข้อมูลสถิติ"]=="ข้อมูลสถิติ"].index)
 df = df.set_index("ข้อมูลสถิติ")
 return df  


#InterestBearingDebt.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

def InterestBearingDebt(symbol,keywordlist = ['เงินกู้','ตราสารหนี้','หุ้นกู้','เงินเบิกเกินบัญชี'],viewlog=True):
 """
    Returns TL,IBD,Log : หนี้สินรวม , หนี้สินที่คาดว่ามีดอกเบี้ย และ Log ของรายการ 
    
    arg1 (string): ตัวย่อของหุ้น
    
    arg2 (list): keywordlist เป็นตัวคัดกรองข้อมูลส่วน keywordlist เป็นตัวคัดกรองข้อมูล

 """   
###### zone1 ดึงข้อมูล
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 symbolMofi = symbol.replace("&","%26")
 #url = "https://www.set.or.th/set/companyhighlight.do?symbol="+symbolMofi
 url = "https://www.set.or.th/set/companyfinance.do?type=balance&symbol="+symbolMofi+"&language=th&country=TH"
 r = requests.get(url,headers)
 soup = BeautifulSoup(r.content, "lxml")

###### zone2 หาส่วนที่จะใช้
 data = soup.find_all("table",{"class":"table table-hover table-info"})
 if(len(data)==0):
    print("ไม่พบข้อมูล")
    return 0,0,0   
 tr = data[2].find_all("tr") 

###### zone3 เตรียมดาต้ามาประมวลผล
 asset = []
 lb = []
 valuelb = []
 eq = []
 g = 0
 rp = []  
 previousKey = ""
 for i in tr:
   td = i.find_all("td") 
   if(len(td)==2):
     if(td[1].text=="สินทรัพย์"):
         g+=1
     if(td[1].text=="หนี้สิน"):
         g+=1
     if(td[1].text=="ส่วนของผู้ถือหุ้น"):
         g+=1    
   if(len(td)>2):
    #print(td[1].text)
    #rp.append(td[1].text)
    m = td[2].text.replace(",","")
    m = float(m)   
    #print(m)
    if("\xa0" in td[1].text[0]):
      #print(td[1].text) 
      pass   
 
    if(g==1):
       asset.append(td[1].text)   
    if(g==2):
       if( (td[1].text!="รวมหนี้สินไม่หมุนเวียน") and (td[1].text!="รวมหนี้สินหมุนเวียน")  ):
        
        if(viewlog):
          print(td[1].text+" "+str(m))   
        
        if("เงินกู้ยืม" in previousKey):
            if(("เงินกู้ยืม" in td[1].text) or ("เงินรับฝาก" in td[1].text)):
              previousKey = "เงินกู้ยืม"
            else:
              previousKey = td[1].text.strip()  
              lb.append(td[1].text)
              valuelb.append(m)
        else:
            previousKey = td[1].text.strip()
            lb.append(td[1].text)
            valuelb.append(m)
          
    if(g==3):
        eq.append(td[1].text)        
        
###### zone4 clean พวกแถวที่จะทำให้รายการคำนวณผิด
 mainlabel = []
 valuemainlabel = []
 mainindex = -1
 sublabel = []
 valuesublabel = []
  
 #rp1 = []
 for i in range(len(lb)-1): # -1 คือเอา รวมหนี้สิน ออก
  #print(lb[i])
  #rp1.append(lb[i])
  if("\xa0" in lb[i][0]):
    #print(lb[i]) 
    sublabel.append("("+str(mainindex)+") "+lb[i])
    valuesublabel.append(valuelb[i]) 
  else:
    mainindex+=1
    mainlabel.append("("+str(mainindex)+") "+lb[i])
    valuemainlabel.append(valuelb[i]) 
    
    
###### zone5 คำนวณผลรวม      
 sumlb = 0
 InterestBearingDebt = 0
 #keywordlist = ['เงินกู้','ตราสารหนี้']
    
 rp1 = []   
 for i in range(len(mainlabel)):
  k = 0
  indexSub = 0
  for j in range(len(sublabel)):
    if(mainlabel[i][0:3] in sublabel[j][0:3]):
      k+=1
      subjective = mainlabel[i]+" "+sublabel[j]
      #print(mainlabel[i]+" "+sublabel[j]+ " "+str(valuesublabel[j]))
      #print(subjective)   
      if any(s in subjective for s in keywordlist):    
         #print(mainlabel[i]+" "+sublabel[j]+ " "+str(valuesublabel[j])) 
         rp1.append(mainlabel[i]+" "+sublabel[j]+ " "+str(valuesublabel[j]))   
         InterestBearingDebt += valuesublabel[j]
      sumlb += valuesublabel[j]
  if(k==0):
    #print(mainlabel[i])# + " "+str(valuemainlabel[i]))
    subjective = mainlabel[i].strip()#+" "+sublabel[j]
    #print(subjective) 
    if any(s in subjective for s in keywordlist):  
       #print(mainlabel[i]+" "+str(valuemainlabel[i])) 
       rp1.append(mainlabel[i]+" "+str(valuemainlabel[i]))
       InterestBearingDebt += valuemainlabel[i]
    sumlb += valuemainlabel[i]
    

###### zone6 clear report    
 for i in range(len(rp1)): 
  rp1[i] = rp1[i].replace("\xa0","")
  rp1[i] = rp1[i].replace("(1)","")
  rp1[i] = rp1[i].replace("(2)","")
  rp1[i] = rp1[i].replace("(3)","")
  rp1[i] = rp1[i].replace("(4)","")
  rp1[i] = rp1[i].replace("(5)","")
  rp1[i] = rp1[i].replace("(6)","")
  rp1[i] = rp1[i].replace("(7)","")
  rp1[i] = rp1[i].replace("(8)","")
  rp1[i] = rp1[i].replace("(9)","")
  rp1[i] = rp1[i].replace("(0)","")
###### zone7 return ผลลัพธ์
 return sumlb,InterestBearingDebt,rp1










#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

def readme():
    with open('README.rst', encoding='utf-8') as f:
        return f.read()

setup(
     name='starfishX',   # This is the name of your PyPI-package.
     version='0.155496',  # Update the version number for new releases
     author='nattapat attiratanasunthron',
     author_email='tapattan@gmail.com',
     url='https://github.com/tapattan/starfishX',
     packages=['starfishX'],
     zip_safe = False,
     keywords=['starfishx','finance','การลงทุน','หุ้น'],
     description='Get data of stock exchange thailand (SET)',
     long_description=readme(),
     long_description_content_type='text/markdown',
     install_requires=['requests','beautifulsoup4','urllib3','pyqt5','pandas','html5lib','ffn','tqdm','seaborn','scipy','sklearn','squarify'], #
)
